/*
 * Copyright (C) 2000-2003 KFA: Andrew Moenk and Alex Scott
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*! \file cuf-global.h
   Misc. Everthing that didn't fit anyware else.
*/

#include <time.h>
#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <sys/stat.h>
#include <sys/types.h>
# ifndef WIN32
#  include <dirent.h>
#  include <unistd.h>
#  include <stdarg.h>
#  include <sys/time.h>
# else
#  include <io.h>
#  include <direct.h>
# endif

#ifdef MEMWATCH
# include "memwatch.h"
#endif

#ifndef _CUF_GLOBAL_H_
# define _CUF_GLOBAL_H_

# ifdef CONFIGURED
//#  include "config.h"
# endif

# ifdef __cplusplus
extern "C" {
# endif


/*** Commonly used macros through out CUF ***/

/*
   Universal TRUE/FALSE for any C compiler.
*/
/*!
   Universal True.
*/
# ifndef TRUE
#  define TRUE			(1 == 1)
# endif
/*!
   Universal False.
*/
# ifndef FALSE
#  define FALSE			(1 == 0)
# endif


/*!
   File error. Used by the global file reading/writing functions.
*/
# ifndef FERR
#  define FERR			(-2)
# endif

/*!
   \param p Target string.
   \param s Source string.
   \desc
   Works like strcat, accept inlined, and if you have a faster memmove then
   it might work better with really large strings.
*/
# ifndef addtext
#  define addtext(p,s) { int plen=strlen (p), slen=strlen (s); memmove (p+plen, s, slen); p[plen+slen]='\0'; }
# endif

/*!
   \param p1 Reciving buffer.
   \param pos Offset to append the other buffer at. Must be a variable
            because it is auto incrimented.
   \param p2 Buffer to copy from.
   \param len Length in bytes to copy data from p2 into p1.
   \desc
   Inserts a buffer into another. No bounds checking is done so this could
     cause a buffer overflow if you underestimate the receiving buffer size.
*/
# ifndef insmem
#  define insmem(p1, pos, p2, len) if (len) memmove (p1+pos, p2, len); pos+=len
# endif

/*!
   \param p1 Data buffer.
   \param pos Offset of data buffer to start copy from. Must be a variable because it is auto incrimented.
   \param p2 Reciving buffer.
   \param len Length in bytes to copy from p1 into p2.
   \desc
   Copies a segment from one buffer into a receiving buffer. Be sure the
     receiving buffer has adequate size to hold the expected length or this
     could cause a buffer overflow.
*/
# ifndef getmem
#  define getmem(p1, pos, p2, len) if (len) memmove (p2, p1+pos, len); pos+=len
# endif

/*
   A Few time saveing if macros
*/
/*!
   \param p Pointer to check.
   \desc
   Checks pointer, if non-null it frees it.
*/
# ifndef iffree
#  define iffree(p) if (p) free (p)
# endif
/*!
   \param p Pointer to check.
   \param f Function to call.
   \desc
   Checks pointer, if non-null it passes it to the specifyed function.
*/
# ifndef iffunc
#  define iffunc(p,f) if (p) f (p)
# endif
/*!
   \param fd File descriptor (Must be of the FILE type).
   \desc
   Checks a FILE descriptor, if non-null it calls fclose() on it.
*/
# ifndef iffclose
#  define iffclose(fd) if (fd) fclose (fd)
# endif

/*!
   \param t The type to allocate.
   \desc
   Type calloc.
*/
# ifndef tcalloc
#  define tcalloc(t) (t *)calloc (1, sizeof(t))
# endif
/*!
   \param t The type to allocate.
   \param n The number of elements in the array.
   \desc
   Type array calloc.
*/
# ifndef tncalloc
#  define tncalloc(t,n) (t *)calloc (n, sizeof(t))
# endif
/*!
   \param len Length of the string to allocate.
   \desc
   String calloc.
*/
# ifndef scalloc
#  define scalloc(len) (char *)calloc (len, sizeof(char))
# endif

/*!
   \desc
   Structured string dup. p is the pointer to the struct. v is the string
     variable name of the struct member and string to be duplicated. Makes
     the code hard to read, probably will drop this macro edventualy.
*/
# ifndef ssdup
#  define ssdup(p,v) p->v=strdup (v)
# endif

/*!
   \param s String.
   \param p Pad character.
   \param len Full length of string+pad.
   \desc
   Pads string 's' with the string 'p' to the length of 'len'.
*/
# ifndef padstring
#  define padstring(s, p, len) { int _i; for (_i=strlen (s); _i < len; _i+=strlen (p)) addtext (s, p); }
# endif

/*!
   \param b The boolean argument to check.
   \desc
   True/False String.
*/
#define tfstr(b)	(b ? "True" : "False")

/*
   To standardize mapping of __IFUNC_ to the various types.
   I dont remember where I pulled this from.
*/
#ifdef __GNUC__
# if __GNUC__ > 2 || \
     (__GNUC__ == 2 && __GNUC_MINOR__ >= (defined __cplusplus ? 6 : 4))
#  define __IFUNC__	__PRETTY_FUNCTION__
# else
#  define __IFUNC__	((__const char *) 0)
# endif
#else
# if defined __STDC_VERSION__ && __STDC_VERSION__ >= 199901L
#  define __IFUNC__ 	__func__
# else
#  define __IFUNC__	((__const char *) 0)
# endif
#endif


/*** Global Enumerations ***/

/*!
   Comparison enumeration.
*/
typedef enum _Comparison {
	CLessThan	=	-1,	//!< A is less than B
	CEqualTo	=	 0,	//!< A is equal to B
	CGreaterThan	=	 1	//!< A is greater than B
} Comparison;


/*** Global Structures ***/

/*!
   Data String.
*/
typedef struct _DString {
	unsigned long len;		//!< Length of data.
	char *data;			//!< Pointer to data.
} DString;


/*** Common function shit^H^H^H^Hcastings. ***/

/*! type name for when calling a void function */
typedef void (*FuncVoid) (void);
/*! type name for when calling a function with a boolean argument */
typedef void (*FuncBool) (int);
/*! type name for when calling a function with a char argument */
typedef void (*FuncChar) (char);
/*! type name for when calling a function with a int argument */
typedef void (*FuncInt) (int);
/*! type name for when calling a function with a long argument */
typedef void (*FuncLong) (long);
/*! type name for when calling a function with a float argument */
typedef void (*FuncFloat) (float);
/*! type name for when calling a function with a double argument */
typedef void (*FuncDouble) (double);
/*! type name for when calling a function with a string argument */
typedef void (*FuncString) (char *);
/*! type name for when calling a function with a #DString argument */
typedef void (*FuncDString) (DString *);

/*! type name for a writing a void (returns TRUE if function is to be called) */
typedef int (*FuncWVoid) (void);
/*! type name for a writing a boolean */
typedef int (*FuncWBool) (void);
/*! type name for a writing a character */
typedef char (*FuncWChar) (void);
/*! type name for a writing a int */
typedef int (*FuncWInt) (void);
/*! type name for a writing a long */
typedef long (*FuncWLong) (void);
/*! type name for a writing a float */
typedef float (*FuncWFloat) (void);
/*! type name for a writing a double */
typedef double (*FuncWDouble) (void);
/*! type name for a writing a string */
typedef char *(*FuncWString) (void);
/*! type name for a writing a #DString */
typedef DString *(*FuncWDString) (void);


/*** Global File read/write Functions ***/

#ifndef getch
char getch (int fd);
char ungetch (int fd);
#endif
char *getline (int fd);
int putch (int fd, char c);
int putchunk (int fd, char *str);
int putline (int fd, char *str);


/*** Wrapers For FILE* Type Operations (WARNING: Untested) ***/

#define fgetch(ff) getch (fileno (ff))
#define fungetch(ff) ungetch (fileno (ff))
#define fgetline(ff) getline (fileno (ff))
#define fputch(ff, c) putch (fileno (ff), c)
#define fputchunk(ff, s) putchunk (fileno (ff), s)
#define fputline(ff, s) putline (fileno (ff), s)


/*** CUF Global Reporting Functions ***/

void cuf_error (char *module, int fatal, char *fmt, ...);
void cuf_warning (char *module, char *fmt, ...);
void cuf_message (char *module, int level, char *fmt, ...);
void cuf_debug_level_set (int level);


/*** Data Strings ***/

DString *dstring_new (unsigned long len);
void dstring_free (DString *dstr);


/*** Miscellaneous ***/

char *get_cwd (void);
# ifdef WIN32
typedef unsigned int mode_t;
# endif
int mkdirp (const char *pathname, mode_t mode);

# ifdef __cplusplus
}
# endif

#endif /* _CUF_GLOBAL_H_ */
