/*
 * Copyright (C) 1999-2003 Andrew Moenk and Alex Scott
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*! \file llist.h
   A link list implementation v3.0
*/

#include "cuf-global.h"

#ifndef _LLIST_H_
#define _LLIST_H_

/*!
   \desc
   The default pivot threshold for #llist_quicksort().
*/
#define DEFAULT_PIVOT_THRESHOLD		20

/*!
   \see _Llist
*/
typedef struct _Llist Llist;

/*!
   \desc
   Linked list speedups, only to be modifyed internaly.
*/
typedef struct _LlistExtras {
	Llist *first;
	Llist *last;
	int done;
	int pivot_threshold;
	unsigned long length;
	int autorewind;
} LlistExtras;

/*!
   \desc
   The Linked List node structure.
   \note
   Its not suggested you access anything but the member 'data' directly.
*/
struct _Llist {
	Llist *prev, *next;		//!< Next and previous nodes.
	LlistExtras *ext;		//!< Extras member, used for internal speedups.
	void *data;			//!< The data pointer.
};

typedef Comparison (*CompareFunc) (void *, void *);
typedef void (*OperationFunc) (void *);


/* Attempt at backwards compatable functions */

void *llist_node_data_next (Llist **inlist);
void *llist_node_data_prev (Llist **inlist);
void *llist_node_data_seek_cfunc (Llist **inlist, void *data,
                                  CompareFunc cfunc);
void *llist_node_data_seek_position (Llist **inlist, unsigned long pos);


/*** Add/Remove data from a list ***/

Llist *llist_append (Llist *list, void *data);
Llist *llist_prepend (Llist *list, void *data);
Llist *llist_remove (Llist *list, void *data);
Llist *llist_remove_cfunc (Llist *list, void *data, CompareFunc cfunc);
Llist *llist_remove_opfunc (Llist *list, void *data, OperationFunc opfunc);
Llist *llist_remove_cfunc_opfunc (Llist *list, void *data, CompareFunc cfunc,
                                  OperationFunc opfunc);


/*** Dupicate/Deallocation of a list ***/

Llist *llist_duplicate (Llist *orglist);
void llist_free (Llist *list);
void llist_free_opfunc (Llist *list, OperationFunc opfunc);


/*** Data Retrieval/List Traversing ***/

Llist *llist_seek (Llist *list, void *data);
Llist *llist_seek_cfunc (Llist *list, void *data, CompareFunc cfunc);
Llist *llist_seek_position (Llist *list, unsigned long pos);
Llist *llist_rewind (Llist *list);
Llist *llist_fastforward (Llist *list);


/*** Sorting/Data Position Manipulation ***/

Llist *llist_reverse (Llist *list);
Llist *llist_bubblesort (Llist *list, CompareFunc cfunc);
Llist *llist_quicksort (Llist *list, CompareFunc cfunc);
Llist *llist_data_rise (Llist *list, void *data);
Llist *llist_data_fall (Llist *list, void *data);


/*** List Option Manipulation ***/

void llist_autorewind_set (Llist *list, int autorewind);
void llist_default_autorewind_set (int autorewind);
void llist_pivot_threshold_set (Llist *list, int pivot_threshold);

/*** Miscellaneous ***/

unsigned long llist_length (Llist *list);
unsigned long llist_length_recalculate (Llist *list);
void llist_extras_print (Llist *list);


/***-- Macros To Lessen The Function Names --***/

/*!
   \param l The list to modify.
   \param d The pointer to append.
   \desc
   Appends a data pointer to the linked list.
   \note
   Their is no indication of failure if you run out of memory.
*/
#define lla(l, d)		l=llist_append (l, d)
/*!
   \param l The list to modify.
   \param d The pointer to append.
   \desc
   Prepends a data pointer to the linked list.
   \note
   Their is no indication of failure if you run out of memory.
*/
#define llp(l, d)		l=llist_prepend (l, d)

/*!
   \param l The list to modify.
   \param d The pointer to remove.
   \desc
   Removes a data pointer from the linked list.
*/
#define llrm(l, d)		l=llist_remove (l, d)
/*!
   \param l The list to modify.
   \param d The pointer to compare.
   \param cf The comparison function to call.
   \desc
   Removes a data pointer from the linked list using the comparison function
     to locate the data to be removed.
*/
#define llrmcf(l, d, cf)		l=llist_remove_cfunc (l, d, (CompareFunc)cf)
/*!
   \param l The list to modify.
   \param d The pointer to remove.
   \param op The operation function to call on the removed data.
   \desc
   Removes a data pointer from the linked list, then calling the operation
     function on the removed data.
*/
#define llrmop(l, d, op)		l=llist_remove_opfunc (l, d, (OperationFunc)op)
/*!
   \param l The list to modify.
   \param d The pointer to compare.
   \param cf The comparison function to call.
   \param op The operation function to call on the removed data.
   \desc
   Removes a data pointer from the linked list using the comparison function
     to locate the data to be removed. Then calling the operation function
     on the removed data.
*/
#define llrmcfop(l, d, cf, op)	l=llist_remove_cfunc_opfunc (l, d, (CompareFunc)cf, (OperationFunc)op)

/*!
   \param l The list to free.
   \desc
   Frees a linked list and all the member nodes.
*/
#define llfree(l)		llist_free (l)
/*!
   \param l The list to modify.
   \param op The operation function to call on each nodes data.
   \desc
   Frees a linked list and all the member nodes. The operation function is
     called on each nodes data as the node is removed.
*/
#define llfreeop(l, op)		llist_free_opfunc (l, (OperationFunc)op)

/*!
   \param l The list to traverse.
   \retval "void *" Of the current nodes data.
   \retval NULL On list end or error.
   \desc
   Returns the data pointed to by the current node then increments the list
     one node.
*/
#define llnext(l)		llist_node_data_next (&l)
/*!
   \param l The list to traverse.
   \retval "void *" Of the current nodes data.
   \retval NULL On list beginning or error.
   \desc
   Returns the data pointed to by the current node then decrements the list
     one node.
*/
#define llprev(l)		llist_node_data_prev (&l)

/*!
   \param l The list to seek in.
   \param d The pointer to compare.
   \param cf The comparison function to call.
   \retval "void *" Of the found data.
   \retval NULL When no match is found or error.
   \desc
   Searches the linked list using the comparison function to locate the
     requested data and returns the found data pointer.
*/
#define llseekcf(l, d, cf)	llist_node_data_seek_cfunc (&l, d, (CompareFunc)cf)
/*!
   \param l The list to seek in.
   \param pos The desired posistion in the list.
   \retval "void *" Of the found data.
   \retval NULL When no match is found or error.
   \desc
   Seeks to the n'th node in the list and returns its data pointer.
*/
#define llseekp(l, pos)	llist_node_data_seek_position (&l, pos)

/*!
   \param l The node to move.
   \desc
   Moves the current node's data tward the \b beginning of the list by one
     node. If this is the first node nothing is done.
*/
#define llrise(l)		l=llist_data_rise (l, l->data)
/*!
   \param l The node to move.
   \desc
   Moves the current node's data tward the \b end of the list by one node.
     If this is the last node nothing is done.
*/
#define llfall(l)		l=llist_data_fall (l, l->data)

/*!
   \param l The list to rewind.
   \desc
   Rewinds the list so as to point at the \b first node.
*/
#define llrew(l)		l=llist_rewind (l)
/*!
   \param l The list to fastworward.
   \desc
   Fastforwards the list so as to point at the \b last node.
*/
#define llff(l)			l=llist_fastforward (l)
/*!
   \param l The list to reverse.
   \desc
   Reverses the contents order of the linked list.
*/
#define llrev(l)		l=llist_reverse (l)

/*!
   \param l The list you want the number of nodes from.
   \retval int With the number of nodes present in the list.
   \retval 0 On error or NULL list.
   \desc
   Retrieves the number of nodes present in the linked list.
*/
#define lllen(l)		llist_length (l)

/*!
   \param l The list you want to traverse.
   \param v the variable to assign the node data to.
   \desc
   Simplifyed for loop syntax to traverse a linked list.

   Example Usage:
   \code
	llforeach (mylist, string) {
		if (*string == '\0') continue;
		printf ("string=%s\n", string);
	}
   \endcode
*/
#define llforeach(l, v)		for (v=llnext (l); v; v=llnext (l))

/*!
   \param l The list you want to traverse.
   \param v The variable to assign the node data to.
   \param b The break condition.
   \desc
   Simplifyed for loop syntax to traverse a linked list. Like #llforeach but
   allows you to specify extra break conditions.

   Example Usage:
   \code
	llforeachb (mylist, string, *string != 'A') {
		if (*string == '\0') continue;
		printf ("string=%s\n", string);
	}
   \endcode
*/
#define llforeachb(l, v, b)	for (v=llnext (l); v && b; v=llnext (l))

#endif /* _LLIST_H_ */
