/*
 * Copyright (C) 1999-2003 Alex Scott
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*! \file plugins.h
   A gerneral purpose plugin loaded/handeler.
*/

#include <dlfcn.h>

#include "llist.h"
#include "array.h"
#include "iparser.h"
#include "plugins_macros.h"

#ifndef _PLUGINS_H_
# define _PLUGINS_H_

/*!
   \desc
   The plugin configuration option type.
*/
typedef enum _PConfOptType {
	PConfBool,			//!< int (0 or 1)
	PConfChar,			//!< char
	PConfInt,			//!< int
	PConfLong,			//!< long
	PConfFloat,			//!< float
	PConfDouble,			//!< double float
	PConfString,			//!< char *
	PConfEnum			//!< Enumerated String list
} PConfOptType;

/*!
   \desc
   A number range structure for plugin configuration. Only to be used
   internaly.
*/
typedef union _NumRange {
	PConfOptType ot;
	/*!
	   \note Nothing to see here
	*/
	struct {
		int minimum, maximum, step, def;
	} pcint;
	/*!
	   \note Nothing to see here
	*/
	struct {
		long minimum, maximum, step, def;
	} pclong;
	/*!
	   \note Nothing to see here
	*/
	struct {
		float minimum, maximum, step, def;
	} pcfloat;
	/*!
	   \note Nothing to see here
	*/
	struct {
		double minimum, maximum, step, def;
	} pcdouble;
} NumRange;

/*!
   \desc
   A plugins' configuration structure. Only to be modifyed internaly. Though
     reading of these variables may be nessary.
*/
typedef struct _PluginConfig {
	char *name;			//!< Name
	char *desc;			//!< Description
	PConfOptType ot;		//!< Option type
	char *def;			//!< Enum list/Bounds/Defaults
	void *data;			//!< The actual/current setting
	void *ref;			//!< Internal reference
} PluginConfig;

/*!
   \desc
   A plugin type structure. Only to be modifyed internaly.
*/
typedef struct _PluginType {
	char *type_name;		// Type specific name
	char *type_desc;		// Type description
	char *info_func_name;		// Info function name
	Llist *plugins;			// Linked list of loaded plugins
} PluginType;

/*!
   \desc
   A plugin structure. Only to be modifyed internaly.
*/
typedef struct _Plugin {
	void *handle;			// Shared object handle
	char *filename;			// File name
	char *name;			// Name of plugin
	char *desc;			// Description of plugin
	unsigned int numofconfig;	// Number of configure options
	PluginConfig *config;		// Configure options
	int (*init) (void);		// Basic initalization func
	void (*cleanup) (void);		// Basic cleanup func
	void *ref;			// Referance to functions structure
} Plugin;

typedef void *(*PluginInfoFunc) (void);


/* Global */

int plugins_object_load (char *filename);
void plugins_scan_dir (char *directory);
void plugins_unload (void);
void plugins_reload (void);
void plugins_free (void);
void plugins_configs_add_all (IOpts *opts);
void plugins_configs_default_all (void);
void plugins_dump_all (void);

/* Types */

PluginType *plugin_type_select (char *type_name);
int plugin_type_register (char *type_name, char *type_desc,
                          char *info_func_name);
void plugin_type_unregister (char *type_name);
void plugin_type_free (PluginType *pt);
void plugin_type_dump (PluginType *pt);
void plugin_type_list_dump (void);


/* Lists */

Llist *plugin_list_get (char *type_name);
char **plugin_list_names_get (char *type_name);
void plugin_list_dump (char *type_name);


/* Plugins  */

Plugin *plugin_select (char *type_name, char *plugin_name);
void plugin_configs_init (Plugin *plugin);
void plugin_configs_free (Plugin *plugin);
void plugin_configs_add (Plugin *plugin, IOpts *opts);
void plugin_configs_default (Plugin *plugin);
void plugin_configs_dump (PluginConfig *pc, int numofconfig);
void plugin_dump (Plugin *plugin);


/* Configuration Helping Functions */

void *plugin_config_data_get (int numofconfs, PluginConfig *pc, char *name);

#endif /* _PLUGINS_H_ */
