/*
 * Copyright (C) 1999-2003 Alex Scott
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*! \file plugins_macros.h
   Helping macros for plugins.

   Please see the example plugin for exact use of these macros.
*/

#include "cuf-global.h"

#ifndef _PLUGINS_MACROS_H_
#define _PLUGINS_MACROS_H_

/*
   For wrapping macros in "'s
*/
#define __str(s)			__strsub(s)
#define __strsub(s)			#s

/*
   For concating macros
*/
#define __cc(sa, sb)			__ccsub(sa, sb)
#define __ccsub(sa, sb)			sa ## sb
#define __cc3(sa, sb, sc)			__cc3sub(sa, sb, sc)
#define __cc3sub(sa, sb, sc)			sa ## sb ## sc

/*
*/
#define __sub(s)			s


/* For Plugin Types */

/*!
   \hideinitializer
   \desc
   This simplifies using the plugin type name as a string.

   You must first define PLUGIN_TYPE_NAME.
*/
#define ptnstr __str(PLUGIN_TYPE_NAME)

/*!
   \hideinitializer
   \desc
   This calls the function plugin_register_type with the correct information.

   You must first define PLUGIN_TYPE_NAME and PLUGIN_TYPE_DESC.
*/
#define plugin_register()		\
plugin_type_register (ptnstr, PLUGIN_TYPE_DESC, "plugin_" ptnstr "_info_get")


/* For Plugins */

/*!
   \hideinitializer
   \desc
   Plugin structure helping macro. Use inside the plugin code its self.
*/
#define PLUGIN_STRUCT			\
Plugin __cc (PLUGIN_NAME, _info) = {\
	NULL,\
	NULL,\
	__str (PLUGIN_NAME),\
	PLUGIN_DESC,\
	PLUGIN_CONFS_NUM,\
	PLUGIN_CONFS,\
	PLUGIN_FUNC_INIT,\
	PLUGIN_FUNC_CLEANUP,\
	&PLUGIN_REF\
};


/*!
   \hideinitializer
   \desc
   Plugin info function helping macro. Use inside the plugin code its self.
*/
#define PLUGIN_FUNC_INFO		\
Plugin *__cc3 (plugin_, PLUGIN_TYPE_NAME, _info_get) (void) {\
	return & __cc (PLUGIN_NAME, _info);\
}\


/*!
   \hideinitializer
   \desc
   Plugin standard calls helping macro. Use inside the plugin code its self.
*/
#define PLUGIN_STD			\
__sub (PLUGIN_STRUCT)\
__sub (PLUGIN_FUNC_INFO)


/*!
   \hideinitializer
   \param n The name of the configuration option.
   \param d The description of the configuration option.
   \param def The default setting.
   \desc
   A simplifyed macro for adding a boolean configuration option.
*/
#define PCAddBool(n, d, def)		\
	{ n, d, PConfBool, (void *)#def, NULL, NULL }

/*!
   \hideinitializer
   \param n The name of the configuration option.
   \param d The description of the configuration option.
   \param def The default setting.
   \desc
   A simplifyed macro for adding a character configuration option.
*/
#define PCAddChar(n, d, def)		\
	{ n, d, PConfChar, (void *)#def, NULL, NULL }

/*!
   \hideinitializer
   \param n The name of the configuration option.
   \param d The description of the configuration option.
   \param t The data type, same as the PConf<*> types with out the 'PConf'.
   \param min The minimum allowed value.
   \param max The maximum allowed value.
   \param s The stepping value.
   \param def The default setting.
   \desc
   A simplifyed macro for adding a number configuration option.
*/
#define PCAddNum(n, d, t, min, max, s, def)\
	{ n, d, PConf ## t, (void *)#min ":" #max ":" #s ":" #def, NULL, NULL }

/*!
   \hideinitializer
   \param n The name of the configuration option.
   \param d The description of the configuration option.
   \param def The default setting.
   \desc
   A simplifyed macro for adding a string configuration option.
*/
#define PCAddString(n, d, def)		\
	{ n, d, PConfString, (void *)def, NULL, NULL }

/*!
   \hideinitializer
   \param n The name of the configuration option.
   \param d The description of the configuration option.
   \param l A comma seprated list of allowed options.
   \param def The default setting.
   \desc
   A simplifyed macro for adding a enumerated list configuration option.
   \note
   This option must be strings.
*/
#define PCAddEnum(n, d, l, def)		\
	{ n, d, PConfEnum, (void *)l ":" def, NULL, NULL }

#define pclen(p)	(sizeof(p)/sizeof(PluginConfig))

/*!
   \hideinitializer
   \param n The name of the configuration option.
   \param t The expected data type.
   \desc
   For use in the plugin source its self. A reduced way to get the
     configuration data.
   \see plugin_config_data_get
*/
#define pcdg(n, t)	(*((t *)plugin_config_data_get (PLUGIN_CONFS_NUM, PLUGIN_CONFS, n)))

/*!
   Data get specificly for strings
*/
#define pcdgs(n)	((char *)plugin_config_data_get (PLUGIN_CONFS_NUM, PLUGIN_CONFS, n))


#endif /* _PLUGINS_MACROS_H_ */
